# Working ORMs

## Getting Started
 
```
cd ./examples/database
npm install
```

### Postgres

To use the examples with a postgres database your first need to create your database.

```
psql -c "CREATE DATABASE nools_example"
```

Now create your tables.

```
psql nools_example -c 'CREATE TABLE "project" ("id" serial PRIMARY KEY, "title" text, "description" text, "createdAt" TIMESTAMP NOT NULL, "updatedAt" TIMESTAMP NOT NULL)'
```

### MySQL

To use the examples with a postgres database your first need to create your database.

```
mysql -e "CREATE DATABASE nools_example"
```

```
mysql -D nools_example -e "CREATE TABLE project (id integer PRIMARY KEY AUTO_INCREMENT, title varchar(255), description varchar(255), createdAt datetime NOT NULL, updatedAt datetime NOT NULL)"
```



## [Patio](//c2fo.github.io/patio)

To run the patio example run `index.js` with the first argument as `patio` and your second argument is the connection string

For example.

```
node index.js patio pg://postgres@127.0.0.1:5432/nools_example
```

Or with MySQL 

```
node index.js patio mysql://root@127.0.0.1:3306/nools_example
```

## [Sequelize](http://sequelizejs.com/)

To run the sequelize example run `index.js` with the first argument as `sequelize` and your second argument is the connection string

For example.

```
node index.js sequelize postgres://postgres@127.0.0.1:5432/nools_example
```

Or with MySQL 

```
node index.js sequelize mysql://root@127.0.0.1:3306/nools_example
```

## The Code

Both of the examples will output the following.

```
ERRORS
{ title: 'Valid Title', description: undefined, id: null } 'description required got: undefined'
{ title: 'Valid Title', description: null, id: null } 'description required got: null'
{ title: 'Valid Title', description: '', id: null } 'description required got: '
{ description: 'Test Description', id: null } 'title required got: undefined'
{ title: null, description: 'Test Description', id: null } 'title required got: null'
{ title: '', description: 'Test Description', id: null } 'title required got: '

VALID MODELS
{"title":"Valid Title","description":"Valid Description","id":null} is valid
```

### The Nools Rules File.

The Rules file located at `model.nools` looks like the following.

```
rule "Project Title Validator" {
    when {
        or(
            p : Project p.title === '',
            p : Project p.title === null,
            p : Project p.title === undefined
        )
    }
    then {
        emit("project-error", {model: p, error: "title required got: " + p.title});
        retract(p);
    }
}

rule "Project Description Validator" {
    when {
        or(
            p : Project p.description === '',
            p : Project p.description === null,
            p : Project p.description === undefined
        )
    }
    then {
        emit("project-error", {model: p, error: "description required got: " + p.description});
        retract(p);
    }
}

rule "Valid Project" {
    //you got here so the model is valid
    salience: -1;
    when {
        p: Project;
    }
    then{
        emit("project-valid", p);
    }
}
```

The rules were intentionally left simple for the sake of clarity. However they represent an example of what you could do with nools and your models.

### The Executing Code.

To run the rules there is a little boiler plate that is used the code is located in a file for each ORM.

`patio_nools.js`

```javascript
"use strict";
var patio = require("patio"),
    nools = require("../../"),
    //patio loads the schema from the database
    Project = patio.addModel("project", {
        plugins: [patio.plugins.TimeStampPlugin] //enable timestamps
    }).timestamp({created: "createdAt", updated: "updatedAt", updateOnCreate: true}),
    //compile your rules file defining your models
    flow = nools.compile(__dirname + "/model.nools", {
        define: {
            Project: Project
        }
    });

function validate(modelsToValidate) {
    var validModels = [], errors = [];
    var session = flow.getSession()
        .on("project-error", function (error) {
            errors.push(error);
        })
        .on("project-valid", function (project) {
            validModels.push(project);
        });
    modelsToValidate.forEach(function (model) {
        session.assert(model);
    });
    return session.match().then(function () {
        return {valid: validModels, errors: errors};
    });
}
```

`sequelize_nools.js`

```javascript
"use strict";
var nools = require("../../"),
    Sequelize = require("sequelize"),
    sequelize = new Sequelize(connectionString),
    Project = sequelize.define('Project', {
        title: Sequelize.STRING,
        description: Sequelize.TEXT
    }, {freezeTableName: true}),
    //compile your rules file specifying your models defined by your ORM
    flow = nools.compile(__dirname + "/model.nools", {
        define: {
            Project: Project.DAO //notice you have to use Project.DAO inorder to specify your model type
        }
    });

function validator(modelsToValidate) {
    //you would typically compile this once at the top but we have different models depending on the ORM
    var validModels = [], errors = [];
    var session = flow.getSession()
        .on("project-error", function (error) {
            errors.push(error);
        })
        .on("project-valid", function (project) {
            validModels.push(project);
        });
    modelsToValidate.forEach(function (model) {
        session.assert(model);
    });
    return session.match().then(function () {
        return {valid: validModels, errors: errors};
    });
}
```

Both ORM integration is very similar with the exception that `sequelize` requires you to use `Project.DAO` in order to get the correct constructor.



